/*
 * Copyright (c) 1997 Pietro Venanzangeli - All Rights Reserved.
 */

/**
 * Realizza un piccolo server interrogabile tramite la porta 23.
 * @author  Pietro Venanzangeli.
 * @version 1.0, 14-Nov-1997
 */

import java.net.*;
import java.io.*;
import LogIn;


class MokaServer {
  /** indica il numero della porta */
  protected static final int SrvPort =23;
  /** indica il tempo in TimeOut per chiudere la connessione */
  protected static final int TimeOUT =10000;
  /**
   * Punto di ingresso del programma Mokaserver.
   * @param String e' la stringa dei comandi presi dall' esterno.
   */
  public static void main(String[] argv) {
    System.out.println("*** MokaServer ***"); // inserisce uno spazio per bellezza di output !!!
    System .out.print(" MokaServer Service\r\n" +
                      " Versione 1.0 \r\n" +
                      " Copyright (c) Pietro Venanzangeli\r\n");
    System.out.println(" Inizializzazione in Corso...");

    /* inizializza la porta del server */
    ServerSocket SRVsocket = null;
    try {
      SRVsocket= new ServerSocket(SrvPort);
      System.out.println(" Telnet Service ready - Host [ " + SRVsocket.getInetAddress().getLocalHost() + " ] ");
    }// verifica se ci sono stati degli errori e in caso gestisce l' eccezione
    catch (IOException err ) {

      System.out.println(" Impossibile aprire la porta " + SrvPort + " - " + err.getMessage());
      System.exit(-1);
    }

    System.out.println(" Server Attivo.");

    while (true) {
      Socket clientSocket = null;
      try {
        clientSocket = SRVsocket.accept();
        clientSocket.setSoTimeout(TimeOUT);

      }
      catch (IOException err ) {
        System.out.println(" Impossibile accettare la connessione -" + err.getMessage());
      }
      /* nuovo theard per il client... */
      new MokaDialog(clientSocket).start();
    }
  }


}

/**
 * Crea un Thread per il Client che richiede una connessione
 */

class MokaDialog extends Thread {
  /** contine il numero di client presenti sul server */
  protected static int NumClient = 0;
  /** Socket del client */
  protected Socket clientSocket = null;
  /** dati utente */
  protected User crUser = new User(); /* contiene tutti i dati per l'utente corrente */

  /**
   * Costruttore della classe MokaDialog
   * @param socket e' il socket aperto verso il client.
   */
  MokaDialog( Socket socket ) {
    clientSocket =  socket;
    crUser.SetIpAddress(clientSocket.getInetAddress());
    System.out.println(" Nuovo Utente :" + " ip = " + crUser.GetIpAddress()  + " Sessioni aperte " + ++NumClient);


  }

  /** 
   * Metodo Run per la gestione dell' utente
   */
  public void run() {

    try {
      //System.gc(); /* garbate collector */
      crUser.SetInBuff (new BufferedReader(new InputStreamReader(clientSocket.getInputStream())));
      crUser.SetOutBuff(new PrintStream(clientSocket.getOutputStream()));
    }
    catch (IOException e) {
      System.out.println(" Impossibile comunicare con il Client.");
      NumClient--;
      return;
    }
    (crUser.GetOutBuff()).print  ("MokaServer\r\n" +
                                  "MokaServer manager\r\n" +
                                  "Versione 1.0 \r\n" +
                                  "Copyright (c) Pietro Venanzangeli\r\n");

    LogIn ControlUser = new LogIn(crUser);
    (crUser.GetOutBuff()).print  ("\r\nBenvenuti.\r\n" +
                                  "SERVER READY.\r\n");
    new ExecCommand(crUser,": ");
    terminate();
  }

  /** 
   * Termina ed esegue tutte le operazioni di disconnessione del client 
   */

  private void terminate() {
    try {
      clientSocket.close();
    }
    catch (IOException e) {
    }
    System.out.println(" Client "+ crUser.GetIpAddress() +" Disconnesso - Sessioni aperte " + --NumClient);
  }
}

